@@ -77,6 +77,20 @@ class TestAPI(dbusmock.DBusTestCase):
         self.dbus_mock.AddMethod('', 'Do', 'si', 's', 'ret = args[0] * args[1]')
         self.assertEqual(self.dbus_test.Do('foo', 3), 'foofoofoo')
 
+    def test_array_arg(self):
+        '''array argument'''
+
+        self.dbus_mock.AddMethod('', 'Do', 'iaou', '',
+                                 '''assert len(args) == 3
+assert args[0] == -1;
+assert args[1] == ['/foo']
+assert type(args[1]) == dbus.Array
+assert type(args[1][0]) == dbus.ObjectPath
+assert args[2] == 5
+''')
+        self.assertEqual(self.dbus_test.Do(-1, ['/foo'], 5), None)
+
+
     def test_methods_on_other_interfaces(self):
         '''methods on other interfaces'''
 
@@ -99,6 +113,31 @@ class TestAPI(dbusmock.DBusTestCase):
         with open(self.mock_log.name) as f:
             self.assertRegex(f.read(), '^[0-9.]+ OtherDo\n[0-9.]+ OtherDo2\n[0-9.]+ OtherDo3$')
 
+    def test_methods_type_mismatch(self):
+        '''calling methods with wrong arguments'''
+
+        def check(signature, args, err):
+            self.dbus_mock.AddMethod('', 'Do', signature, '', '')
+            try:
+                self.dbus_test.Do(*args)
+                self.fail('method call did not raise an error for signature "%s" and arguments %s'
+                          % (signature, args))
+            except dbus.exceptions.DBusException as e:
+                self.assertTrue(err in str(e), e)
+
+        # not enough arguments
+        check('i', [], 'TypeError: More items found')
+        check('is', [1], 'TypeError: More items found')
+
+        # too many arguments
+        check('', [1], 'TypeError: Fewer items found')
+        check('i', [1, 'hello'], 'TypeError: Fewer items found')
+
+        # type mismatch
+        check('u', [-1], 'convert negative value to unsigned')
+        check('i', ['hello'], 'TypeError: an integer is required')
+        check('s', [1], 'TypeError: Expected a string')
+
     def test_add_object(self):
         '''add a new object'''
 
@@ -162,7 +201,7 @@ class TestAPI(dbusmock.DBusTestCase):
         obj1 = self.dbus_con.get_object('org.freedesktop.Test', '/obj1')
 
         self.assertEqual(obj1.Do0(), 42)
-        self.assertEqual(obj1.Do1(), 31337)
+        self.assertEqual(obj1.Do1(1), 31337)
         self.assertRaises(dbus.exceptions.DBusException,
                           obj1.Do2, 31337)
 
@@ -265,6 +304,10 @@ class TestAPI(dbusmock.DBusTestCase):
             self.dbus_mock.EmitSignal('org.freedesktop.Test.Sub',
                                       'SigTwoArgs',
                                       'su', ['hello', 42])
+            self.dbus_mock.EmitSignal('org.freedesktop.Test.Sub',
+                                      'SigTypeTest',
+                                      'iuvao',
+                                      [-42, 42, dbus.String('hello', variant_level=1), ['/a', '/b']])
 
         caught = []
         ml = GObject.MainLoop()
@@ -272,7 +315,7 @@ class TestAPI(dbusmock.DBusTestCase):
         def catch(*args, **kwargs):
             if kwargs['interface'].startswith('org.freedesktop.Test'):
                 caught.append((args, kwargs))
-            if len(caught) == 2:
+            if len(caught) == 3:
                 # we caught everything there is to catch, don't wait for the
                 # timeout
                 ml.quit()
@@ -299,6 +342,51 @@ class TestAPI(dbusmock.DBusTestCase):
         self.assertEqual(caught[1][1]['path'], '/')
         self.assertEqual(caught[1][1]['interface'], 'org.freedesktop.Test.Sub')
 
+        # check data types in SigTypeTest
+        self.assertEqual(caught[2][1]['member'], 'SigTypeTest')
+        self.assertEqual(caught[2][1]['path'], '/')
+        args = caught[2][0]
+        self.assertEqual(args[0], -42)
+        self.assertEqual(type(args[0]), dbus.Int32)
+        self.assertEqual(args[0].variant_level, 0)
+
+        self.assertEqual(args[1], 42)
+        self.assertEqual(type(args[1]), dbus.UInt32)
+        self.assertEqual(args[1].variant_level, 0)
+
+        self.assertEqual(args[2], 'hello')
+        self.assertEqual(type(args[2]), dbus.String)
+        self.assertEqual(args[2].variant_level, 1)
+
+        self.assertEqual(args[3], ['/a', '/b'])
+        self.assertEqual(type(args[3]), dbus.Array)
+        self.assertEqual(args[3].variant_level, 0)
+        self.assertEqual(type(args[3][0]), dbus.ObjectPath)
+        self.assertEqual(args[3][0].variant_level, 0)
+
+    def test_signals_type_mismatch(self):
+        '''emitting signals with wrong arguments'''
+
+        def check(signature, args, err):
+            try:
+                self.dbus_mock.EmitSignal('', 's', signature, args)
+                self.fail('EmitSignal did not raise an error for signature "%s" and arguments %s'
+                          % (signature, args))
+            except dbus.exceptions.DBusException as e:
+                self.assertTrue(err in str(e), e)
+
+        # not enough arguments
+        check('i', [], 'TypeError: More items found')
+        check('is', [1], 'TypeError: More items found')
+
+        # too many arguments
+        check('', [1], 'TypeError: Fewer items found')
+        check('i', [1, 'hello'], 'TypeError: Fewer items found')
+
+        # type mismatch
+        check('u', [-1], 'convert negative value to unsigned')
+        check('i', ['hello'], 'TypeError: an integer is required')
+        check('s', [1], 'TypeError: Expected a string')
 
 if __name__ == '__main__':
     # avoid writing to stderr
